/*-----------------------------*/
/* AutoGuider            V2.00 */
/*-----------------------------*/
#include <MsTimer2.h>

#define LED   13

// MOTOR1 = RA
#define CLK_RA      5
#define RESET_RA    3
#define ENABLE_RA   4
#define CWCCW_RA   19
#define TQ1_RA      6
#define TQ2_RA      7

// MOTOR2 = DEC
#define CLK_DEC    10
#define RESET_DEC   8
#define ENABLE_DEC  9
#define CWCCW_DEC   2
#define TQ1_DEC    11
#define TQ2_DEC    12

#define SW1        14
#define SW2        15

#define EXT_SW_RA_CCW    14
#define EXT_SW_RA_CW     15
#define EXT_SW_DEC_CCW   17
#define EXT_SW_DEC_CW    18
#define EXT_SW_FAST      16

#define MOTOR_RA   1
#define MOTOR_DEC  2

#define SW_ON      LOW
#define SW_OFF     HIGH

#define MOTOR_STOP 0
#define MOTOR_RUN  1

#define CCW        HIGH
#define CW         LOW

#define COMMAND_NORTH       1
#define COMMAND_SOUTH       2
#define COMMAND_EAST        3
#define COMMAND_WEST        4
#define COMMAND_MOVE        10
#define COMMAND_MOVE_NORTH  11
#define COMMAND_MOVE_SOUTH  12
#define COMMAND_MOVE_EAST   13
#define COMMAND_MOVE_WEST   14
#define COMMAND_QUIT        20
#define COMMAND_QUIT_NORTH  21
#define COMMAND_QUIT_SOUTH  22
#define COMMAND_QUIT_EAST   23
#define COMMAND_QUIT_WEST   24
#define COMMAND_OTHER       0

#define MAX_TORQUE 100
#define MIN_TORQUE 25

#define MOTOR_SPEED_NORMAL  500
#define MOTOR_SPEED_FAST    100

int motor_ra_stat  = MOTOR_STOP;
int motor_dec_stat = MOTOR_STOP;
int motor_rotate   = CCW;

void setup()   {                
  pinMode(LED, OUTPUT);  

  pinMode(CLK_RA,    OUTPUT);     
  pinMode(RESET_RA,  OUTPUT);     
  pinMode(ENABLE_RA, OUTPUT);     
  pinMode(CWCCW_RA,  OUTPUT);
  pinMode(TQ1_RA,    OUTPUT);
  pinMode(TQ2_RA,    OUTPUT);

  pinMode(CLK_DEC,    OUTPUT);     
  pinMode(RESET_DEC,  OUTPUT);     
  pinMode(ENABLE_DEC, OUTPUT);     
  pinMode(CWCCW_DEC,  OUTPUT);
  pinMode(TQ1_DEC,    OUTPUT);
  pinMode(TQ2_DEC,    OUTPUT);

  pinMode(SW1, INPUT);
  pinMode(SW2, INPUT);
  
  pinMode(EXT_SW_RA_CCW, INPUT );
  pinMode(EXT_SW_RA_CW,  INPUT );
  pinMode(EXT_SW_DEC_CCW,INPUT );
  pinMode(EXT_SW_DEC_CW, INPUT );
  
  stop_motor( MOTOR_RA  );
  stop_motor( MOTOR_DEC );

  init_motor( MOTOR_RA  );
  init_motor( MOTOR_DEC );
  
  set_torque( MIN_TORQUE, MOTOR_RA  );
  set_torque( MIN_TORQUE, MOTOR_DEC );

  MsTimer2::set(1,motor_run);   // 1msec
  MsTimer2::start();
  
  Serial.begin(9600);
}

void motor_run()
{
  if( motor_ra_stat  == MOTOR_RUN ) {
    digitalWrite(CLK_RA, HIGH);
  }
  if( motor_dec_stat == MOTOR_RUN ) {
    digitalWrite(CLK_DEC, HIGH);
  }
  delayMicroseconds(10);
  digitalWrite(CLK_RA, LOW);
  digitalWrite(CLK_DEC, LOW);
}

void loop()                     
{
  int command_no;
  
  if (Serial.available() > 0) {
    command_no = get_command();
    exec_command( command_no );
  }
  
  if( digitalRead(EXT_SW_RA_CCW)  == SW_ON ) {
    motor_run_manual( EXT_SW_RA_CCW  );
  }
  if( digitalRead(EXT_SW_RA_CW)   == SW_ON ) {
    motor_run_manual( EXT_SW_RA_CW   );
  }
  if( digitalRead(EXT_SW_DEC_CCW) == SW_ON ) {
    motor_run_manual( EXT_SW_DEC_CCW );
  }
  if( digitalRead(EXT_SW_DEC_CW)  == SW_ON ) {
    motor_run_manual( EXT_SW_DEC_CW  );
  }  
}

void motor_run_manual( int sw_no ) {
  MsTimer2::stop();
  switch( sw_no ) {
    case EXT_SW_RA_CCW:
      digitalWrite(CWCCW_RA,CCW);
      set_torque( MAX_TORQUE, MOTOR_RA );
      break;
    case EXT_SW_RA_CW:
      digitalWrite(CWCCW_RA,CW);
      set_torque( MAX_TORQUE, MOTOR_RA );
      break;
    case EXT_SW_DEC_CCW:
      digitalWrite(CWCCW_DEC,CCW);
      set_torque( MAX_TORQUE, MOTOR_DEC );
      break;
    case EXT_SW_DEC_CW:
      digitalWrite(CWCCW_DEC,CW);
      set_torque( MAX_TORQUE, MOTOR_DEC );
      break;
  }    

  while( digitalRead(sw_no) == SW_ON ) {
    switch( sw_no ) {
      case EXT_SW_RA_CCW:
      case EXT_SW_RA_CW:
        if( digitalRead(EXT_SW_FAST) == SW_ON ) {
          delayMicroseconds(MOTOR_SPEED_FAST  );
        } else {
          delayMicroseconds(MOTOR_SPEED_NORMAL);
        }
        digitalWrite(CLK_RA, HIGH);
        delayMicroseconds(10);
        digitalWrite(CLK_RA, LOW);
        break;
      case EXT_SW_DEC_CCW:
      case EXT_SW_DEC_CW:
        if( digitalRead(EXT_SW_FAST) == SW_ON ) {
          delayMicroseconds(MOTOR_SPEED_FAST  );
        } else {
          delayMicroseconds(MOTOR_SPEED_NORMAL);
        }
        digitalWrite(CLK_DEC, HIGH);
        delayMicroseconds(10);
        digitalWrite(CLK_DEC, LOW);
        break;
    }
  }

  switch( sw_no ) {
    case EXT_SW_RA_CCW:
    case EXT_SW_RA_CW:
      set_torque( MIN_TORQUE, MOTOR_RA );
      break;
    case EXT_SW_DEC_CCW:
    case EXT_SW_DEC_CW:
      set_torque( MIN_TORQUE, MOTOR_DEC );
      break;
  }  
  MsTimer2::start();  
}

void exec_command( int command_no )
{
  switch( command_no ) {
    case COMMAND_MOVE_NORTH:
      set_torque( MAX_TORQUE, MOTOR_DEC );
      digitalWrite(CWCCW_DEC,CW );
      motor_dec_stat = MOTOR_RUN;
      break;
    case COMMAND_MOVE_SOUTH:
      set_torque( MAX_TORQUE, MOTOR_DEC );
      digitalWrite(CWCCW_DEC,CCW);
      motor_dec_stat = MOTOR_RUN;
      break;
    case COMMAND_MOVE_EAST:
      set_torque( MAX_TORQUE, MOTOR_RA );   
      digitalWrite(CWCCW_RA,CW );
      motor_ra_stat = MOTOR_RUN;
      break;
    case COMMAND_MOVE_WEST:
      set_torque( MAX_TORQUE, MOTOR_RA );    
      digitalWrite(CWCCW_RA,CCW);
      motor_ra_stat = MOTOR_RUN;
      break;
    case COMMAND_QUIT_NORTH:
      set_torque( MIN_TORQUE, MOTOR_DEC );    
      motor_dec_stat = MOTOR_STOP;
      break;
    case COMMAND_QUIT_SOUTH:
      set_torque( MIN_TORQUE, MOTOR_DEC );    
      motor_dec_stat = MOTOR_STOP;
      break;
    case COMMAND_QUIT_EAST:
      set_torque( MIN_TORQUE, MOTOR_RA );    
      motor_ra_stat = MOTOR_STOP;
      break;
    case COMMAND_QUIT_WEST:
      set_torque( MIN_TORQUE, MOTOR_RA );    
      motor_ra_stat = MOTOR_STOP;
      break;
    case COMMAND_QUIT:
      motor_ra_stat  = MOTOR_STOP;
      motor_dec_stat = MOTOR_STOP;
      break;
    default:
      break;
  }
}

int get_command()
{
  char rbyte = 0;
  int command_no = 0;
  
  while(true) {
    if (Serial.available() > 0) {
      rbyte = Serial.read();
      if ( rbyte == '#' ) {
        break;
      }
      if ( rbyte == ':' ) {
        command_no = 0;
      }
      switch( rbyte ) {
        case 'M': command_no =  COMMAND_MOVE;  break;
        case 'Q': command_no =  COMMAND_QUIT;  break;
        case 'n': command_no += COMMAND_NORTH; break;
        case 's': command_no += COMMAND_SOUTH; break;
        case 'e': command_no += COMMAND_EAST;  break;
        case 'w': command_no += COMMAND_WEST;  break;
      }
    }
  }

  return command_no;
}  

void init_motor(int motor_No)
{
  switch( motor_No ) {
    case 1:
      digitalWrite( RESET_RA,  LOW );
      digitalWrite( ENABLE_RA, LOW );
      digitalWrite( CWCCW_RA,  CCW );
      delayMicroseconds(100);
      digitalWrite( ENABLE_RA, HIGH );
      delayMicroseconds(100);
      digitalWrite( RESET_RA,  HIGH );
      delayMicroseconds(100);
      break;
    case 2:
      digitalWrite( RESET_DEC,  LOW );
      digitalWrite( ENABLE_DEC, LOW );
      digitalWrite( CWCCW_DEC,  CCW );
      delayMicroseconds(100);
      digitalWrite( ENABLE_DEC, HIGH );
      delayMicroseconds(100);
      digitalWrite( RESET_DEC,  HIGH );
      delayMicroseconds(100);
      break;
  }
}

void stop_motor( int motor_No )
{
  switch( motor_No ) {
    case 1:
      digitalWrite( RESET_RA,  LOW );
      digitalWrite( ENABLE_RA, LOW );
      break;
    case 2:
      digitalWrite( RESET_DEC,  LOW );
      digitalWrite( ENABLE_DEC, LOW );   
      break;
  }
}

void set_torque( int torq, int motor_no )
{
   switch( motor_no ) {
    case 1:
      switch( torq ) {
        case 100: digitalWrite( TQ1_RA, LOW  ); digitalWrite( TQ2_RA, LOW  ); break;
        case  75: digitalWrite( TQ1_RA, HIGH ); digitalWrite( TQ2_RA, LOW  ); break;
        case  50: digitalWrite( TQ1_RA, LOW  ); digitalWrite( TQ2_RA, HIGH ); break;
        case  25: digitalWrite( TQ1_RA, HIGH ); digitalWrite( TQ2_RA, HIGH ); break;
      }
      break;
    case 2:
      switch( torq ) {
        case 100: digitalWrite( TQ1_DEC, LOW  ); digitalWrite( TQ2_DEC, LOW  ); break;
        case  75: digitalWrite( TQ1_DEC, HIGH ); digitalWrite( TQ2_DEC, LOW  ); break;
        case  50: digitalWrite( TQ1_DEC, LOW  ); digitalWrite( TQ2_DEC, HIGH ); break;
        case  25: digitalWrite( TQ1_DEC, HIGH ); digitalWrite( TQ2_DEC, HIGH ); break;
      }
      break;
  } 
}

//------ E N D   O F   P R O G R A M -------------------------------------------------

